/*
 * Template for developing attacks in Java (Windows, or Unix in FIFO mode)
 *
 * Template for DPA contest v4.2 AES-128 Improved RSM implementation
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class Attack {
    // TODO: modify, if necessary, the number of the attacked subkey
    // See the FAQ on the DPA contest website for more information
    // about the subkey numbering
    final static int ATTACKED_SUBKEY = 0;

    final static int NUM_SAMPLES = 1704402;

    String fifoInName, fifoOutName;
    FileInputStream fifoIn;
    FileOutputStream fifoOut;

    int numTraces;

    byte[] plaintext;
    byte[] ciphertext;
    byte[] shuffle0;
    byte[] shuffle10;
    byte[] offsets;
    float[] samples;
    byte[][] result;


    public Attack(String fifoInName, String fifoOutName) {
	this.fifoInName = fifoInName;
	this.fifoOutName = fifoOutName;

	plaintext = new byte[16];
	ciphertext = new byte[16];
	shuffle0 = new byte[16];
	shuffle10 = new byte[16];
	offsets = new byte[16];
	samples = new float[NUM_SAMPLES];
	result = new byte[16][256];
    }


    public void launchAttack() throws Exception {
	// Open the FIFOs
	FileInputStream fifoIn = new FileInputStream(new File(fifoInName));
	FileOutputStream fifoOut = new FileOutputStream(new File(fifoOutName));

	// Read the number of traces
	numTraces = readInt(fifoIn);

	// Send the we are ready signal
	byte[] start = new byte[]{10, 46, 10};
	fifoOut.write(start);
	fifoOut.flush();

	// Main loop
	for(int traceNum = 0; traceNum <= numTraces; traceNum++) {
	    // Read the plaintext, ciphertext and offset
	    readFifo(fifoIn, plaintext);
	    readFifo(fifoIn, ciphertext);
	    readFifo(fifoIn, shuffle0);
	    readFifo(fifoIn, shuffle10);
	    readFifo(fifoIn, offsets);

	    // Read the samples
	    for (int sampleNum = 0; sampleNum < NUM_SAMPLES; sampleNum++) {
		float sample = readFloat(fifoIn);
		samples[sampleNum] = sample;
	    }


	    // *******************************
	    // TODO: put your attack code here
	    // *******************************
	    // Your attack code can use:
	    // - plaintext: the plaintext
	    // - ciphertext: the ciphertext
	    // - offsets: the offsets (0 unless the wrapper is launched with --provide_offsets_v4_2)
	    // - shuffle0: the Shuffle0 permutation (0 unless the wrapper is launched with --provide_offsets_v4_2)
	    // - shuffle10: the Shuffle10 permutation (0 unless the wrapper is launched with --provide_offsets_v4_2)
	    // - samples: the samples of the trace
	    //
	    // And must fill the result bi-dimensional (16*256) array
	    // For i in [0..15], results[i] contains all the 256 possible values for the byte
	    // i of the attacked subkey sorted according to their probability (index 0: most probable,
	    // index 255: least probable)


	    // Send the result to the wrapper
	    fifoOut.write(ATTACKED_SUBKEY);
	    for(int i = 0; i < 16; i++) fifoOut.write(result[i]);
	    fifoOut.flush();
	}

	// Close the FIFOs
	fifoIn.close();
	fifoOut.close();
    }


    private int readInt(FileInputStream in) throws Exception {
	byte[] t = new byte[4];
	readFifo(in, t);
	return ByteBuffer.wrap(t).order(ByteOrder.LITTLE_ENDIAN).getInt();
    }

    private float readFloat(FileInputStream in) throws Exception {
	byte[] t = new byte[4];
	readFifo(in, t);
	return ByteBuffer.wrap(t).order(ByteOrder.LITTLE_ENDIAN).getFloat();
    }


    private void readFifo(FileInputStream in, byte [] dest) throws Exception {
	int toRead, bytesRead;

	toRead = dest.length;
	bytesRead = 0;

	while (bytesRead < toRead) {
	    bytesRead += in.read(dest, bytesRead, toRead - bytesRead);
	}
    }


    public static void main(String[] args) throws Exception {
	if(args.length < 2) {
	    System.out.println("java Attack <fifo_in> <fifo_out>\n");
	    System.out.println("On Windows, the fifo are:");
	    System.out.println("\\\\.\\pipe\\fifo_from_wrapper and \\\\.\\pipe\\fifo_to_wrapper");
	    System.out.println("On Linux, the fifo are:");
	    System.out.println("fifo_from_wrapper and fifo_to_wrapper");
	} else {
	    Attack attack = new Attack(args[0], args[1]);
	    attack.launchAttack();
	}
    }

}
