#include <stdint.h>
#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <iomanip>
#include <vector>
#include <string>

uint32_t total_num_traces;
uint32_t current_trace_num;

const std::string fifo_in_name = "fifo_from_wrapper";
const std::string fifo_out_name = "fifo_to_wrapper";

FILE *fifo_in;
FILE *fifo_out;


class Trace_v4_RSM
{
public:
  static const unsigned int plaintext_length;
  static const unsigned int ciphertext_length;
  static const unsigned int num_samples;

  std::vector<uint8_t> plaintext;
  std::vector<uint8_t> ciphertext;
  std::vector<int8_t> samples;
  uint8_t offset;

  Trace_v4_RSM ();

  bool read_trace_from_fifo ();
};

class AES_Attack_Partial_Results
{
public:
  uint8_t targeted_subkey_num;
  std::vector<std::vector<uint8_t> > partial_results;

  AES_Attack_Partial_Results ();

  bool send_results_to_fifo ();
};


Trace_v4_RSM trace;
AES_Attack_Partial_Results results;



const unsigned int Trace_v4_RSM::plaintext_length = 16;
const unsigned int Trace_v4_RSM::ciphertext_length = 16;
const unsigned int Trace_v4_RSM::num_samples = 435002;

Trace_v4_RSM::Trace_v4_RSM ()
: plaintext (plaintext_length, 0),
  ciphertext (ciphertext_length, 0),
  samples (num_samples, 0)
{

}


bool Trace_v4_RSM::read_trace_from_fifo ()
{
  int8_t buf;

  // Plaintext
  for (unsigned int i = 0; i < plaintext_length; i++)
    {
      fread (&buf, 1, 1, fifo_in);
      plaintext[i] = (uint8_t) buf;
    }

  // Ciphertext
  for (unsigned int i = 0; i < ciphertext_length; i++)
    {
      fread (&buf, 1, 1, fifo_in);
      ciphertext[i] = (uint8_t) buf;
    }

  // Offset
  fread (&buf, 1, 1, fifo_in);
  offset = (uint8_t) buf;

  // Samples
  for (unsigned int i = 0; i < num_samples; i++)
    {
      fread (&buf, 1, 1, fifo_in);
      samples[i] = buf;
    }

  return true;
}


std::ostream & operator<< (std::ostream &os, const Trace_v4_RSM &t)
{
  // Plaintext
  os << "Plaintext: ";
  for (unsigned int i = 0; i < t.plaintext.size (); i++)
    os << std::hex << std::setw (2) << std::setfill ('0') << (unsigned int) t.plaintext[i];
  os << std::endl;

  // Ciphertext
  os << "Ciphertext: ";
  for (unsigned int i = 0; i < t.ciphertext.size (); i++)
    os << std::hex << std::setw (2) << std::setfill ('0') << (unsigned int) t.ciphertext[i];
  os << std::endl;

  // Offset
  os << "Offset: ";
  os << std::hex << std::setw (1) << (unsigned int) t.offset;
  os << " ";

  // Samples
  os << "Samples: ";
  for (unsigned int i = 0; i < t.samples.size (); i++)
    os << std::dec << (int) t.samples[i] << std::endl;

  os << std::endl;

  return os;
}


AES_Attack_Partial_Results::AES_Attack_Partial_Results ()
  : partial_results(16)
{
  for (unsigned int index = 0; index < partial_results.size (); index++)
    partial_results[index].resize (256);

  targeted_subkey_num = 0;
}


bool AES_Attack_Partial_Results::send_results_to_fifo ()
{
  int8_t buf;
  
  // Subkey
  buf = targeted_subkey_num;
  fwrite (&buf, 1, 1, fifo_out);

  // Results
  for (unsigned int byte = 0; byte < 16; byte++)
    for (unsigned int val = 0; val < 256; val++)
      {
	buf = partial_results[byte].at (val);
	fwrite (&buf, 1, 1, fifo_out);
      }

  fflush (fifo_out);

  return true;
}


bool read_num_traces ()
{
  fread (&total_num_traces, sizeof (total_num_traces), 1, fifo_in);

  return true;
}


bool send_start_attack ()
{
  int8_t start[] = {0x0A, 0x2E, 0x0A};
  fwrite (start, 3, 1, fifo_out);

  fflush (fifo_out);

  return true;
}


int main (int argc, char **argv)
{
  // Open the FIFOs
  fifo_in = fopen (fifo_in_name.c_str (), "rb");
  fifo_out = fopen (fifo_out_name.c_str (), "wb");

  if ((fifo_in == NULL) || (fifo_out == NULL))
    exit (EXIT_FAILURE);

  // Read the number of traces
  read_num_traces ();

  // We are ready
  send_start_attack ();

  for (current_trace_num = 0; current_trace_num < total_num_traces; current_trace_num++)
    {
      // Read a trace
      trace.read_trace_from_fifo ();

      // Perform the attack and set the results
      for (unsigned int byte = 0; byte < 16; byte++)
	for (unsigned int val = 0; val < 256; val++)
	  results.partial_results[byte][val] = val;
	  
      // Send the results
      results.send_results_to_fifo ();
    }

  fclose (fifo_in);
  fclose (fifo_out);

  exit (EXIT_SUCCESS);
}
