/*
 * Classes for management of the different editions of the contest
 *
 * Copyright (C) 2013 Guillaume Duc <guillaume.duc@telecom-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef EDITIONS_H
#define EDITIONS_H

#include <stdint.h>
#include <string>

#include "traces.h"
#include "results.h"
#include "comm.h"


class Contest
{
protected:
  uint32_t contest_identifier;
  std::string trace_directory;
  std::string results_file;
  bool allow_overwrite_results_file;

  unsigned int trace_raw_length;
  unsigned int results_raw_length;

  int num_iterations;
  int key_num;

  Trace_Manager * trace_manager;
  Results_Manager * results_manager;
  Attack_Comm * attack_comm;

public:
  Contest (uint32_t contest_identifier,
	   const std::string &trace_directory,
	   const std::string &results_file,
	   bool allow_overwrite_results_file);

  virtual bool run_contest () = 0;
  virtual int set_num_iterations (unsigned int num_iterations) = 0;
  virtual bool set_key_num (unsigned int key_num) = 0;
};


class Contest_v2 : public Contest
{
protected:
  std::string index_file;

  int8_t * trace_buffer;
  int8_t * results_buffer;

  bool comm_compatibility_v2;

public:
  static const uint32_t contest_v2_id;

  Contest_v2 (const std::string &trace_directory,
	      const std::string &index_file,
	      const std::string &results_file,
	      bool allow_overwrite_results_file,
	      bool fork_mode,
	      const std::string &attack_program_name,
	      bool comm_compatibility_v2 = false,
	      bool check_traces_availability = true);

  virtual bool run_contest ();
  virtual int set_num_iterations (unsigned int num_iterations);
  virtual bool set_key_num (unsigned int key_num);
};


class Contest_v4_RSM : public Contest
{
protected:
  std::string index_file;

  int8_t * trace_buffer;
  int8_t * results_buffer;

  bool provide_offset;

  Contest_v4_RSM (uint32_t contest_identifier,
		  const std::string &trace_directory,
		  const std::string &results_file,
		  bool allow_overwrite_results_file);

public:
  static const uint32_t contest_v4_RSM_id;

  Contest_v4_RSM (const std::string &trace_directory,
		  const std::string &index_file,
		  const std::string &results_file,
		  bool allow_overwrite_results_file,
		  bool fork_mode,
		  const std::string &attack_program_name,
		  bool provide_offset,
		  bool check_traces_availability = true);

  virtual bool run_contest ();
  virtual int set_num_iterations (unsigned int num_iterations);
  virtual bool set_key_num (unsigned int key_num);
};


class Contest_v4_2 : public Contest_v4_RSM
{
public:
  static const uint32_t contest_v4_2_id;

  Contest_v4_2 (const std::string &trace_directory,
		const std::string &index_file,
		const std::string &results_file,
		bool allow_overwrite_results_file,
		bool fork_mode,
		const std::string &attack_program_name,
		bool provide_offset,
		bool check_traces_availability = true,
		bool use_float_format = true);
};


#endif /* EDITIONS_H */
