/*
 * Classes for management of the results of an attack
 *
 * Copyright (C) 2013 Guillaume Duc <guillaume.duc@telecom-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef RESULTS_H
#define RESULTS_H

#include <stdint.h>

#include <vector>
#include <iostream>
#include <string>

#include "traces.h"


/*
 * Partial Results
 */
class Attack_Partial_Results
{
public:
  virtual unsigned int get_raw_length () const = 0;
  virtual void set_from_raw (const int8_t *buffer) = 0;
};


class AES_Attack_Partial_Results : public Attack_Partial_Results
{
public:
  uint8_t targeted_subkey_num;
  std::vector<std::vector<uint8_t> > partial_results;

  AES_Attack_Partial_Results ();

  virtual unsigned int get_raw_length () const;
  virtual void set_from_raw (const int8_t *buffer);
};

std::ostream & operator<< (std::ostream &os, const AES_Attack_Partial_Results &r);


/*
 * Results manager
 */
class Results_Manager
{
public:
  virtual unsigned int get_results_raw_length () const = 0;
  virtual bool save_raw_results (const int8_t *buffer,
				 const Trace &trace,
				 uint32_t duration_ms) = 0;

  virtual bool prepare_results_file (const std::string &filename,
				     bool allow_overwrite = false) = 0;
  virtual void close_results_file () = 0;

  virtual void set_number_of_traces (uint32_t number_of_traces) = 0;

  virtual const std::string get_progression_string () const = 0;
};


class Results_Manager_AES : public Results_Manager
{
  static const uint32_t binary_format_version;
  static const uint8_t aes_subkey_length;

  uint32_t contest_identifier;
  uint32_t number_of_traces;

  AES_Attack_Partial_Results partial_results;

  std::string results_filename;
  FILE* results_file;

  bool headers_written;

  int aes;

  std::vector<uint8_t> correct_subkey;

  bool write_headers (const Trace &trace);

public:
  Results_Manager_AES (uint32_t contest_identifier, int aes);

  virtual unsigned int get_results_raw_length () const;
  virtual bool save_raw_results (const int8_t *buffer,
				 const Trace &trace,
				 uint32_t duration_ms);

  virtual bool prepare_results_file (const std::string &filename,
				     bool allow_overwrite = false);
  virtual void close_results_file ();

  virtual void set_number_of_traces (uint32_t number_of_traces);

  virtual const std::string get_progression_string () const;
};

#endif /* RESULTS_H */
