/*
 * Misc functions used by the attack wrapper
 *
 * Copyright (C) 2010 Guillaume Duc <guillaume.duc@telecom-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef UTILS_H
#define UTILS_H

#include <stdint.h>

#include <vector>
#include <string>

#ifdef _WIN32
#include <windows.h>
#else /* _WIN32 */
#include <sys/time.h>
#endif /* _WIN32 */



// TODO: make it more reliable if necessary
#ifdef _WIN32
#define PATHSEP "\\"
#else /* _WIN32 */
#define PATHSEP "/"
#endif /* _WIN32 */

/*
 * Convert an hexadecimal string to a vector of uint8_t (2 hex digits -> uint8_t)
 */
void hex2uint8_vector (const std::string &hex, std::vector<uint8_t> * vect);

/*
 * Convert an hexadecimal string to a vector of uint8_t (1 hex digits -> uint8_t)
 */
void hex2uint4_vector (const std::string &hex, std::vector<uint8_t> * vect);

/*
 * Check whether a file exists
 */
bool file_exists (const std::string &filename);

/*
 * Check whether a file exists
 */
bool file_exists (const std::string &dir, const std::string &filename);

/*
 * Check whether a file exists
 */
bool file_exists (const std::string &dir, const std::string &sub_dir,
		  const std::string &filename);


/**
 * Class timer
 */
class Timer
{
#ifdef _WIN32
  DWORD start_tick;
#else /* _WIN32 */
  struct timeval start_time;
  struct timeval stop_time;
#endif /* _WIN32 */

public:
  void start ();
  uint32_t stop ();
};


#endif /* UTILS_H */
